'Imports Win
Imports System.IO

Module Module1
    Private Confirm As Boolean = True
    Private ShowStuff As Boolean = True
    Private Benchmark As Boolean = False
    Private Benchmarknums As Int16 = 1
    'Private DeleteSrc As Boolean

    Private StartTime As Decimal
    Private EndTime As Decimal

    Dim InPath$, OutFile$, Path$, RPGName$

    Sub Main()
        Dim i%

        'This just does the command line stuff
        ParseCommand()

        ' ShowStuff will be true by default

        ' If it's true, then we can dump the header
        If ShowStuff Then
            Console.WriteLine("O.H.R.RPG.C.E RPG File Relumper")
            Console.WriteLine("Just try and pronounce that!")
            Console.WriteLine("Relumper (C)2004 Mike Caron")
        End If

        ' If they gave us the /l option, then InPath will already be
        '  set. We don't have to prompt them in this case.
        If InPath = "" Then
            Console.WriteLine("Please enter the path to the folder with the unlumped RPG:")
            Console.WriteLine("(Leave blank to quit)")
            InPath = Console.ReadLine.Trim
        End If

        ' If they opted to leave it blank, get outta here.
        If InPath = "" Then Exit Sub

        ' Here, we extract the destination name of the RPG. It's generally
        '  the same name as the folder, so I assume that.
        ' Perhaps there should be a /c option, to allow this... hmm...
        RPGName = Mid(InPath, InPath.LastIndexOf("\") + 2)

        ' Tack on a slash at the end, if neccessary.
        ' Side note: I really really really really like the VB.Net syntax
        ' for doing this! :D
        If Not InPath.EndsWith("\") Then InPath &= "\"

        ' Ditto for this. /o will have already set it, if included
        If Path = "" Then
            Console.WriteLine("Please enter the path to the folder which should contain the lumped RPG:")
            Console.WriteLine("(Leave blank for default folder: " & Environment.CurrentDirectory & ")")
            Path = Console.ReadLine.Trim
        End If

        ' However, a blank will set the current directory, instead of
        '  quitting.
        If Path = "" Then
            Path = Environment.CurrentDirectory
        End If

        ' Again, stick on a slash.
        If Not Path.EndsWith("\") Then Path &= "\"

        ' OutFile is a convenience variable that points to the full
        '  output path.
        OutFile = Path & RPGName & ".rpg"

        ' yn will hold the result of the user confirmation
        Dim yn As String

        ' If the user supplied /y, Confirm will be false, thus skipping
        '  this whole section.
        If Confirm Then
            Console.WriteLine()
            Console.WriteLine("Lumping " & InPath & " into " & OutFile)
            Console.Write("OK? (Y/N) ")
            yn = Console.ReadLine

            ' I could have enforced Y or N, but I decided that I'd
            '  be lenient. As a result, anything but a "Y" or "y" will 
            '  will quit
            If yn.ToLower <> "y" Then
                Exit Sub
            End If
        End If
        'End

        ' Used for timing
        StartTime = Timer

        ' This is another cool feature of VB.Net. MUCH better than the
        '  old method.
        Try
            ' This loop is run whether /b is set or not. The main difference
            '  is that if they don't set /b, then Benchmarknums defaults to
            '  1, which makes it only run once.
            For i = 1 To Benchmarknums
                If Benchmark And ShowStuff Then
                    Console.WriteLine("#" & i & "...")
                End If
                ' Lump is the main proc that does the grunt work.
                Lump(InPath, OutFile, ShowStuff And Not Benchmark)
            Next
        Catch ex As Exception
            ' Ah crap. Dump the error, get out.
            If ShowStuff Then Console.Error.WriteLine("Error while lumping!")
            If ShowStuff Then Console.Error.WriteLine(ex.Message)
            Exit Sub
        End Try

        ' If we got here, then everything went ok.
        EndTime = Timer

        ' Ask whether to delete the source folder, if neccessary.
        '        If DeleteSrc Then
        '            If Confirm Then
        '                Console.WriteLine()
        '                Console.WriteLine("Deleting " & InPath)
        '                Console.Write("OK? (Y/N) ")
        '                yn = Console.ReadLine

        '                ' I could have enforced Y or N, but I decided that I'd
        '                '  be lenient. As a result, anything but a "Y" or "y" will 
        '                '  will quit
        '                If yn.ToLower <> "y" Then
        '                    ' Nope, skip ahead.
        '                    GoTo nodelete
        '                End If
        '            End If

        '            RmDir(InPath)
        '        End If
        'nodelete:
        ' Dump a success message
        If ShowStuff Then Console.WriteLine("'" & OutFile & "' successfully created!")

        'Calculate and dump the time it took.
        If ShowStuff Then Console.WriteLine("Took " & Math.Round((EndTime - StartTime) / Benchmarknums, 3) & " Seconds!")
    End Sub

    Function LngSwapBytes(ByVal num As Integer) As Int32
        Dim tmp As String, tmp2 As String, i As Integer
        Dim byts() As Byte, byts2() As Byte

        ' This function will take a sane byte-order Integer, and
        '  turn it into the insane pseudo-byte-swapped order required
        '  by the OHR. But, it only needs it in the lump header.
        ' Elsewhere in the data, it's sane. ARGH!

        ReDim byts(3)
        byts = BitConverter.GetBytes(num)

        ReDim byts2(3)
        byts2(2) = byts(0)
        byts2(3) = byts(1)
        byts2(0) = byts(2)
        byts2(1) = byts(3)

        LngSwapBytes = BitConverter.ToInt32(byts2, 0)
        'LngSwapBytes = num
    End Function

    ' GetQToken(strIn$, strSep$) as String
    ' Use: Tokenize a string
    ' strIn: The string to tokenize
    ' strSep: The token seperators
    ' Notes: - GetQToken is supposed to recognize quotes, but it doesn't yet.
    '          - Edit: It now does quotes!
    '        - Use it recursively, passing "" to strIn after the first call.
    '          - Edit: Now both params are optional, so you can just go: a = GetQToken

    Public Function GetQToken(Optional ByVal strIn$ = "", Optional ByVal strSep$ = " ", Optional ByVal pe As Boolean = False) As String
        On Error Resume Next
        Static chunks$()
        Static pos&
        Dim olpos&
        If pe Then olpos = pos
        If strIn = "" Then
            pos = pos + 1
            If UBound(chunks) < pos Then GetQToken = "" Else GetQToken = chunks(pos)

            If GetQToken.StartsWith("""") Then
                Do Until GetQToken.EndsWith("""")
                    pos += 1
                    GetQToken &= chunks(pos)
                Loop

                GetQToken = GetQToken.Substring(1, GetQToken.Length - 2)
            End If
        Else
            chunks = Split(strIn, strSep)
            pos = 0
            GetQToken = chunks(pos)
            If GetQToken.StartsWith("""") Then
                Do Until GetQToken.EndsWith("""")
                    pos += 1
                    GetQToken &= chunks(pos)
                Loop
            End If
        End If
        If pe Then pos = olpos
    End Function

    Public Sub StrtoBytes(ByRef ab() As Byte, ByVal Str As String)
        Dim i&

        ' This just takes a string, and turns it into a byte array
        ' suitable for use in a lump name.
        ReDim ab(Len(Str) - 1)

        For i = 1 To Len(Str)
            ab(i - 1) = Asc(Mid(Str, i, 1))
        Next

    End Sub

    Public Function FileArray(ByVal sPath As String, Optional ByVal attrib As Microsoft.VisualBasic.FileAttribute = FileAttribute.Normal)
        Dim fls() As String, a As String

        ' This function is a duplicate of a function in VBCore.dll.
        ' But, I didn't want and depenancies, so I was forced to make
        '  my own function.

        ' Oh, right. Description. It takes a folder path, and returns
        '  a string array with the names of all files within. It can
        '  also filter by attributes.

        a = Dir(sPath, attrib)
        ReDim fls(0)
        Do Until a = ""
            fls(UBound(fls)) = a
            a = Dir()
            ReDim Preserve fls(UBound(fls) + 1)
        Loop
        ReDim Preserve fls(UBound(fls) - 1)
        FileArray = fls
    End Function

    Public Function ExistFile(ByVal sPath$) As Boolean
        Dim a&

        ' Another function from VBCore. Checks to see if the given
        '  file is existant or not. However, the implementation is
        '  much different from the VBCore version. The old source is
        '  included below the real function.

        ExistFile = File.Exists(sPath)

        ' --- Old function ---
        'ExistFile = True
        'Try
        '    a = FileLen(sPath)
        'Catch
        '    ExistFile = False
        'End Try
    End Function

    Sub ParseCommand()
        Dim a$

        ' This just parses the command line. A baby should be able
        '  to understand it. If you can't understand it, get a baby!

        'Dim cnt%
        a = GetQToken(Command, " ")

        Do While a <> ""
            Select Case LCase(a)
                Case "/y" 'No confirmations
                    Confirm = False
                Case "/l" 'The lump contents
                    InPath = GetQToken()
                Case "/o" 'Output Path
                    Path = GetQToken()
                Case "/s" 'Suppress extraneous crap
                    ShowStuff = False
                Case "/b" 'Benchmark
                    Benchmark = True
                    Benchmarknums = GetQToken()
                    'Case "/d" 'delete source
                    '    DeleteSrc = True
                Case "/?" 'Help
                    Help()
            End Select
            a = GetQToken()
        Loop
    End Sub

    Sub Lump(ByVal InF$, ByVal OutF$, Optional ByVal wr As Boolean = True)
        Dim tmp() As Byte, tmpstr() As Byte, curfile As String
        Dim files() As String

        ' This here function is the meat and potatoes of the program.
        ' It takes a folder to lump, and an output name to write.
        ' It then does it.

        ' Check to see if the output file is already there.
        ' If we don't delete it, the OHR will have problems trying to
        '  read it if the new lumped file is shorter than the old one.
        If File.Exists(OutFile) Then File.Delete(OutFile)

        ' Get the list of files in the input folder. folder.
        files = FileArray(InPath & "*.*")


        ' The streams are cool, becuase I could hook up a filter if I
        '  choose. But, I won't because I have no need to do so.

        ' instr is the stream used to read a file in.
        ' outstr is the stream that is connected to the output file.
        Dim instr As Stream, outstr As Stream

        ' Open the output stream.
        outstr = File.OpenWrite(OutFile)

        ' Initialize the array.
        ReDim tmp(0)

        'Loop through the list of files.
        For Each curfile In files
            ' Turn the filename into a byte array...
            StrtoBytes(tmp, CStr(curfile))

            ' ... and write it to the disk.
            outstr.Write(tmp, 0, tmp.Length)

            ' OHR strings are null terminated.
            outstr.WriteByte(0)

            ' Get the length, swap it, turn it into a byte array, and write it to disk.
            outstr.Write(BitConverter.GetBytes(LngSwapBytes(FileLen(InPath & curfile))), 0, 4)
            ' It's not null terminated. (that would be dumb!)

            ' Init the array again, we need it still.
            ReDim tmp(FileLen(InPath & curfile) - 1)

            ' Open the input stream
            instr = File.OpenRead(InPath & curfile)

            ' Read the whole file
            instr.Read(tmp, 0, tmp.Length)

            ' Close the input stream
            instr.Close()

            ' Write the whole file to the output
            outstr.Write(tmp, 0, tmp.Length)

            ' wr is a flag that tells us whether to write a status message
            ' ShowStuff = false, or Benchmark = true will turn this off.
            ' (In fact, that's pretty much the logic used...)
            If wr Then Console.WriteLine(curfile & ": " & tmp.Length & " bytes")
        Next

        ' Close the output stream, we're done.
        outstr.Close()
    End Sub

    Sub Help()
        ' Write help. This sub consists of only Console.WriteLine.
        With Console.Out
            .WriteLine("relump [/? | [/l <Unlumped Folder>] [/o <Output Folder>] [/y] [/s] [/b <Number of times>]]")
            .WriteLine("/? : Shows this help message.")
            .WriteLine("/l : Indicates the folder to lump")
            .WriteLine("/o : The folder to put the lumped file")
            .WriteLine("/y : Suppresses confirmation")
            .WriteLine("/s : Suppresses extraneous output")
            .WriteLine("/b : Benchmark mode. Lumps folder multiple times, for speed testing")
            '.WriteLine("/d : Deletes the source folder afterwards.")
            .WriteLine()
            .WriteLine("If you do not include /l or /o, the program will ask you for them.")
            .WriteLine()
            .WriteLine("Questions? Comments? Flames? My email: pkmnfrk@finalfantasyq.com")
        End With
        End
    End Sub
End Module

